package test;

import static org.quartz.CronScheduleBuilder.*;
import static org.quartz.JobBuilder.*;
import static org.quartz.SimpleScheduleBuilder.*;
import static org.quartz.TriggerBuilder.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.quartz.Job;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.SchedulerFactory;
import org.quartz.Trigger;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.matchers.GroupMatcher;

public class QuartzUtil {
	private static final String GROUP = "stdgroup";
	private static final String JOB_SUFFIX = "_job";
	private static final String TRIGGER_SUFFIX = "_trigger";
	private static Scheduler getScheduler() throws SchedulerException {
		SchedulerFactory schedFact = new StdSchedulerFactory();
		Scheduler sched = schedFact.getScheduler();
		return sched;  
	}
	private static void run(String name, Class<? extends Job> jobclz, Trigger trigger) throws SchedulerException {
		Scheduler sched = getScheduler();
		JobDetail job = newJob(jobclz).withIdentity(name + JOB_SUFFIX, GROUP).build();
		sched.scheduleJob(job, trigger);
	}
	public static void runNow(String name, Class<? extends Job> jobclz) throws SchedulerException {
		Trigger trigger = newTrigger().withIdentity(name + TRIGGER_SUFFIX, GROUP).startNow().build();
		run(name, jobclz, trigger);
	}
	public static void runEveryDay(String name, Class<? extends Job> jobclz, int h, int m) throws SchedulerException {
		Trigger trigger = newTrigger().withIdentity(name + TRIGGER_SUFFIX, GROUP).startNow().withSchedule(dailyAtHourAndMinute(h, m)).build();
		run(name, jobclz, trigger);
	}
	public static void runInterval(String name, Class<? extends Job> jobclz, int m) throws SchedulerException {
		Trigger trigger = newTrigger().withIdentity(name + TRIGGER_SUFFIX, GROUP).startNow().withSchedule(simpleSchedule().withIntervalInMinutes(m).repeatForever()).build();
		run(name, jobclz, trigger);
	}
	public static class JobInfo {
		private String name;
		private String clznam;
		private Date lastrun;
		private Date nextrun;
		public JobInfo(String name, String clznam, Date lastrun, Date nextrun) {
			super();
			this.name = name;
			this.clznam = clznam;
			this.lastrun = lastrun;
			this.nextrun = nextrun;
		}
		public String getName() {
			return name;
		}
		public String getClznam() {
			return clznam;
		}
		public Date getLastrun() {
			return lastrun;
		}
		public Date getNextrun() {
			return nextrun;
		}
	}
	public static List<JobInfo> getList() throws SchedulerException {
		List<JobInfo> res = new ArrayList<JobInfo>();
		Scheduler sched = getScheduler();
		for(JobKey jk : sched.getJobKeys(GroupMatcher.jobGroupEquals(GROUP))) {
			for(Trigger tg : sched.getTriggersOfJob(jk)) {
				res.add(new JobInfo(jk.getName(), sched.getJobDetail(jk).getJobClass().getName(), tg.getPreviousFireTime(), tg.getNextFireTime()));
			}
		}
		return res;
	}
}
