<?php
class T1 {
    public $f1;
    public $f2;
    public function __construct($f1, $f2) {
        $this->f1 = $f1;
        $this->f2 = $f2;
    }
}

interface T1DatabaseAccess {
    public function get_one($f2);
    public function get_all();
    public function put($o);
    public function remove($f1);
}

class T1DatabaseAccessMysqli implements T1DatabaseAccess {
    private $server;
    private $username;
    private $password;
    private $database;
    public function __construct($server, $username, $password, $database) {
        $this->server = $server;    
        $this->username = $username;    
        $this->password = $password;    
        $this->database = $database;    
    }
    private function get_connection() {
        //$con = new mysqli($this->server, $this->username, $this->password, $this->database);
        $con = new mysqli();
        if(mysqli_connect_errno()) {
            die(mysqli_connect_error());
        }
        return $con;
    }
    public function get_one($f2) {
        $con = $this->get_connection();
        $stmt = $con->prepare('SELECT F1,F2 FROM T1 WHERE F2=?') or die(mysqli_error($con));
        $stmt->bind_param('s', $f2);
        $stmt->execute() or die(mysqli_error($con));
        $rs = $stmt->get_result();
        if($row = $rs->fetch_array(MYSQLI_ASSOC)) {
            $res = new T1($row['F1'], $row['F2']);  
        } else {
            die("$f2 not found");
        }
        $stmt->close();
        $con->close();
        return $res;
    }
    public function get_all() {
        $con = $this->get_connection();
        $stmt = $con->prepare('SELECT F1,F2 FROM T1') or die(mysqli_error($con));
        $stmt->execute() or die(mysqli_error($con));
        $rs = $stmt->get_result();
        $res = array();
        while($row = $rs->fetch_array(MYSQLI_ASSOC)) {
            $res[] = new T1($row['F1'], $row['F2']);
        }
        $stmt->close();
        $con->close();
        return $res;
    }
    public function put($o) {
        $con = $this->get_connection();
        $stmt = $con->prepare('INSERT INTO T1(F1,F2) VALUES(?,?)') or die(mysqli_error($con));
        $stmt->bind_param('is', $o->f1, $o->f2);
        $stmt->execute() or die(mysqli_error($con));
        $stmt->close();
        $con->close();
    }
    public function remove($f1) {
        $con = $this->get_connection();
        $stmt = $con->prepare('DELETE FROM T1 WHERE F1=?') or die(mysqli_error($con));
        $stmt->bind_param('i', $f1);
        $stmt->execute() or die(mysqli_error($con));
        $stmt->close();
        $con->close();
    }
}

class T1DatabaseAccessFactory {
    const MYSQLI = 'mysqli';
    public static function get_database_access($id, $server, $username, $password, $database) {
        if($id === T1DatabaseAccessFactory::MYSQLI) {
            return new T1DatabaseAccessMysqli($server, $username, $password, $database);
        } else {
            throw new Exception("Unknown T1DatabaseAccess id: $id");
        }
    }
}

interface T1Presenter {
    public function display($data);
}

class T1PresenterHtmlTable implements T1Presenter {
    public function display($data) {
        $rows = "";
        foreach($data as $row) {
            $f1 = $row->f1;
            $f2 = $row->f2;
            $rows .= sprintf("<tr>\r\n<td>$f1</td>\r\n<td>$f2</td>\r\n</tr>\r\n");
        }
        return "<table border='yes'>\r\n<tr>\r\n<th>F1</th>\r\n<th>F2</th>\r\n</tr>\r\n$rows</table>\r\n";   
    }
}

class T1PresenterFactory {
    const HTML_TABLE = 'HTML Table';
    public static function get_presenter($id) {
        if($id === T1PresenterFactory::HTML_TABLE) {
            return new T1PresenterHtmlTable()   ;       
        } else {
            throw new Exception("Unknown T1Presenter id: $id");
        }
    }
}

function test() {
    $data_access = T1DatabaseAccessFactory::get_database_access(T1DatabaseAccessFactory::MYSQLI, 'localhost', 'root', '', 'Test');
    $presenter = T1PresenterFactory::get_presenter(T1PresenterFactory::HTML_TABLE);
    $f2 = $data_access->get_one('BB')->f1;
    echo "$f2<br>\r\n";
    $data = $data_access->get_all();
    echo $presenter->display($data);
    $data_access->put(new T1(999, 'XXX'));
    $data = $data_access->get_all();
    echo $presenter->display($data);
    $data_access->remove(999);
    $data = $data_access->get_all();
    echo $presenter->display($data);
}

test();
?>
