<?php
class T1 {
    public $f1;
    public $f2;
    public function __construct($f1, $f2) {
        $this->f1 = $f1;
        $this->f2 = $f2;
    }
}

interface T1DatabaseAccess {
    public function get_one($f2);
    public function get_all();
    public function put($o);
    public function remove($f1);
}

class T1DatabaseAccessPDO implements T1DatabaseAccess {
    private $constr;
    private $username;
    private $password;
    public function __construct($constr, $username, $password) {
        $this->constr = $constr;    
        $this->username = $username;    
        $this->password = $password;    
    }
    private function get_connection() {
        //$con = new PDO($this->constr, $this->username, $this->password);
        $con = new PDO('java:comp/env/jdbc/MySqlTest');
        $con->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        //$con->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
        return $con;
    }
    public function get_one($f2) {
        $con = $this->get_connection();
        $stmt = $con->prepare('SELECT F1,F2 FROM T1 WHERE F2=:F2');
        $stmt->execute(array(':F2' => $f2));
        if($row = $stmt->fetch()) {
            $res = new T1($row['F1'], $row['F2']);  
        } else {
            die("$f2 not found");
        }
        return $res;
    }
    public function get_all() {
        $con = $this->get_connection();
        $stmt = $con->prepare('SELECT F1,F2 FROM T1');
        $stmt->execute(array());
        $res = array();
        while($row = $stmt->fetch()) {
            $res[] = new T1($row['F1'], $row['F2']);
        }
        return $res;
    }
    public function put($o) {
        $con = $this->get_connection();
        $stmt = $con->prepare('INSERT INTO T1(F1,F2) VALUES(:F1,:F2)');
        $stmt->execute(array(':F1' => $o->f1, ':F2' => $o->f2));
    }
    public function remove($f1) {
        $con = $this->get_connection();
        $stmt = $con->prepare('DELETE FROM T1 WHERE F1=:F1');
        $stmt->execute(array(':F1' => $f1));
    }
}

class T1DatabaseAccessFactory {
    const PDO = 'PDO';
    public static function get_database_access($id, $constr, $username, $password) {
        if($id === T1DatabaseAccessFactory::PDO) {
            return new T1DatabaseAccessPDO($constr, $username, $password);
        } else {
            throw new Exception("Unknown T1DatabaseAccess id: $id");
        }
    }
}

interface T1Presenter {
    public function display($data);
}

class T1PresenterHtmlTable implements T1Presenter {
    public function display($data) {
        $rows = "";
        foreach($data as $row) {
            $f1 = $row->f1;
            $f2 = $row->f2;
            $rows .= sprintf("<tr>\r\n<td>$f1</td>\r\n<td>$f2</td>\r\n</tr>\r\n");
        }
        return "<table border='yes'>\r\n<tr>\r\n<th>F1</th>\r\n<th>F2</th>\r\n</tr>\r\n$rows</table>\r\n";   
    }
}

class T1PresenterFactory {
    const HTML_TABLE = 'HTML Table';
    public static function get_presenter($id) {
        if($id === T1PresenterFactory::HTML_TABLE) {
            return new T1PresenterHtmlTable()   ;       
        } else {
            throw new Exception("Unknown T1Presenter id: $id");
        }
    }
}

function test() {
    $data_access = T1DatabaseAccessFactory::get_database_access(T1DatabaseAccessFactory::PDO, 'mysql:host=localhost;dbname=Test', 'root', '');
    $presenter = T1PresenterFactory::get_presenter(T1PresenterFactory::HTML_TABLE);
    $f2 = $data_access->get_one('BB')->f1;
    echo "$f2<br>\r\n";
    $data = $data_access->get_all();
    echo $presenter->display($data);
    $data_access->put(new T1(999, 'XXX'));
    $data = $data_access->get_all();
    echo $presenter->display($data);
    $data_access->remove(999);
    $data = $data_access->get_all();
    echo $presenter->display($data);
}

test();
?>
